#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Feb  7 14:27:01 2019

@author: havard
"""

import numpy as np
import pandas as pd
import math
import matplotlib.pyplot as plt
import sqlite3 as sql
import time
import datetime
import calendar
import seaborn as sns
import matplotlib.pyplot as plt
import sklearn.cluster as cluster
from mpl_toolkits.basemap import Basemap

    

c1 = 'steelblue'
c2 = 'darkorange'
c3 = 'lightseagreen'
c1l = 'lightblue'
c2l = 'bisque'
c3l = 'paleturquoise'
        

#Validate that MMSI number not is changing
def validateIMO(df5):
    changes = []
    vessels = df5.userid.unique()
    for vessel in vessels:
        imoA = df5[df5.userid == vessel].imo.iloc[0]
        imoB = df5[df5.userid == vessel].imo.iloc[-1]
        if imoA != imoB:
            changes.append(vessel)

    return changes

#Hversinse Distanse: input = point
def haversine(lon1, lat1, lon2, lat2): # FINN KILDE!!!
    """
    Calculate the great circle distance between two points 
    on the earth (specified in decimal degrees)
    """
    # convert decimal degrees to radians 
    lon1, lat1, lon2, lat2 = map(math.radians, [lon1, lat1, lon2, lat2])
    # haversine formula 
    dlon = lon2 - lon1 
    dlat = lat2 - lat1 
    a = math.sin(dlat/2)**2 + math.cos(lat1) * math.cos(lat2) * math.sin(dlon/2)**2
    c = 2 * math.asin(math.sqrt(a)) 
    # Radius of earth in kilometers is 6371
    km = 6371* c
    return km

#Haversine Distances: input = list
def haversineDF(lon1, lat1, lon2, lat2): 
    """
    Calculate the great circle distance between two points 
    on the earth (specified in decimal degrees)
    """
    
    # convert decimal degrees to radians 
    lon1, lat1, lon2, lat2 = map(np.radians, [lon1, lat1, lon2, lat2])
    # haversine formula 
    dlon = lon2 - lon1 
    dlat = lat2 - lat1 
    a = np.sin(dlat/2)**2 + np.cos(lat1) * np.cos(lat2) * np.sin(dlon/2)**2
    c = 2 * np.arcsin(np.sqrt(a)) 
    # Radius of earth in kilometers is 6371
    km = 6371* c
    return km

#Give new operational status
def lableStatus(df,ports):
    tic = time.time()
    
    #Function for finding the distance to the closes port
    def distToPorts(lon,lat,dfPorts):
        dist=list()
        for row in dfPorts.itertuples():
            dist.append(haversine(lon,lat,row[4],row[3]))
        return dist
    
    #Funciton for assigning operatioanl status
    def statusLable(row,dfPorts):
        if row['sog']>3:
            return 1
        else:
            if row['sog']<1:
                if row['nav_status'] == 5:
                    return 2
                if row['nav_status'] == 1:
                    return 3
                
            dist = distToPorts(row['longitude'],row['latitude'],dfPorts)
            d = min(dist)
            if d<1.4:
                return 2
            elif d<45 and row['sog']<3:
                return 3
            else:
                return 1    
                
    #Assign status as new colums
    df['status'] = df.apply(statusLable,args=(ports,),axis=1)
    
    print('Lableing Time: ',time.time()-tic)
    return df

def statusChange(df):
    changes = 0
    
    for row in df.itertuples():
        if row.status ==1 and row.nav_status != 0:
            changes = changes + 1
        elif row.status == 2 and row.nav_status != 5:
            changes = changes +1
        elif row.status == 3 and row.nav_status != 1:
            changes = changes + 1
    
    return changes



#Use new operational Status to lable each leg,port and wait
def legNumbering(df):
    vessels = df.userid.unique()
    df['Leg'] = 0
    df['Port'] = 0
    df['Wait'] = 0
    #Help function for lable Leg nr
    def legList(df):
        leg = []
        legNr = 1
        temp = 2
        for row in df.itertuples():
            if row.status == 1:
                leg.append(legNr)
                temp = legNr
            else:
                leg.append(0)
                if temp == legNr:
                    legNr +=1
        return leg
    #Help function for lable port nr
    def portList(df):
        port = []
        portNr = 1
        temp = 2
        for row in df.itertuples():
            if row.status == 2:
                port.append(portNr)
                temp = portNr
            else:
                port.append(0)
                if temp == portNr:
                    portNr +=1
        return port
    #Help function for lable port nr
    def waitList(df):
        wait = []
        waitNr = 1
        temp = 2
        for row in df.itertuples():
            if row.status == 3:
                wait.append(waitNr)
                temp = waitNr
            else:
                wait.append(0)
                if temp == waitNr:
                    waitNr +=1
        return wait
                
    #Loop through all vessel and lable legs, port and wait number
    for vessel in vessels:
        legL = legList(df[df.userid == vessel])
        df.loc[df.userid == vessel,'Leg']=legL
        
        portL = portList(df[df.userid == vessel])
        df.loc[df.userid == vessel,'Port']=portL
        
        waitL = waitList(df[df.userid == vessel])
        df.loc[df.userid == vessel,'Wait']=waitL
        
    return df
        
#Lable Static MessageTypes with Draught Ratio
def lableStatDraughtRatio(df5,shipList):
    
    #Lable Message Type 5 with draught ratio
    def designDraught(row,df):
        if df[df['MMSI']==row['userid']].size >0:
            desD = df[df['MMSI']==row['userid']].Draught.values[0]
            return row.draught/desD  
        else:
            return 0
    df5['dRatio'] = df5.apply(designDraught,args=(shipList,),axis=1)
    return df5

#Calculate Time Sail, Port, Wait
def timeAnalytics(df,dfSail,dfPort,dfWait):

    vessels = df['userid'].unique()
    voyageTimes = []
    waitingTimes = []
    portTimes = []
    vesselListS = []
    vesselListW = []
    vesselListP = []
    
    for vessel in vessels:
        dfTemp=df[df.userid == vessel] 
        #Sailing
        dfT = dfTemp[dfTemp.Leg != 0]
        times = (dfT.groupby('Leg').last()['unixtime']-dfT.groupby('Leg').first()['unixtime'])/(60*60)
        voyageTimes.extend(times)
        vesselListS.extend([vessel]*len(times))
        
        #Waiting
        dfT = dfTemp[dfTemp.Wait != 0]
        times = (dfT.groupby('Wait').last()['unixtime']-dfT.groupby('Wait').first()['unixtime'])/(60*60)
        waitingTimes.extend(times)
        vesselListW.extend([vessel]*len(times))
        
        #Port
        dfT = dfTemp[dfTemp.Port != 0]
        times = (dfT.groupby('Port').last()['unixtime']-dfT.groupby('Port').first()['unixtime'])/(60*60)
        portTimes.extend(times)
        vesselListP.extend([vessel]*len(times))
    
    dfSail['userid']= vesselListS
    dfSail['voyageTime'] = voyageTimes
    
    dfWait['userid'] = vesselListW
    dfWait['waitingTime'] = waitingTimes
    
    dfPort['userid'] = vesselListP
    dfPort['portTime'] = portTimes
    
    return dfSail, dfPort, dfWait


def legAnalytics(df,df5,dfSail):
    vessels = df['userid'].unique()
    
    maxMessageIntervall = []
    voyageDistance = []
    dRatio = []
    area = []
    heading = []
    speed = []
    
    for vessel in vessels:
        dfTemp=df[df.userid == vessel]          
        dfT = dfTemp[dfTemp.Leg != 0]
        
        #Find each leg's max message intervall
        intervalls = dfT.groupby('Leg').unixtime.apply(lambda x: x.diff().max())/(60*60)
        maxMessageIntervall.extend(intervalls)
        
        #Sailinglenngts  
        voyageDistance.extend(dfT.groupby('Leg').apply(lambda x:haversineDF(x.longitude,\
                    x.latitude,x.longitude.shift(),x.latitude.shift()).sum()))
        
        #DraughtRatio
        startTimes = dfT.groupby('Leg').first()['unixtime']
        endTimes = dfT.groupby('Leg').last()['unixtime']
        for i in range(1,len(startTimes)+1):
            dRatio.extend([df5[(df5.userid == vessel) & (df5.unixtime>=startTimes[i])\
                             & (df5.unixtime<=endTimes[i])]['dRatio'].median()])
    
        #Operational Area    
        area.extend(dfT.groupby('Leg').apply(lambda x: x.Area.iloc[len(x)//2]))
        #Heading
        heading.extend(dfT.groupby('Leg').apply(lambda x: x.cog.iloc[len(x)//2]))
        #Avg Speed
        speed.extend(dfT.groupby('Leg').sog.mean())
        
        
    dfSail['maxMsgInt'] = maxMessageIntervall
    dfSail['voyageDist'] = voyageDistance
    dfSail['dRatio'] = dRatio
    dfSail['dir'] = heading
    dfSail['area'] = area
    dfSail['avgSpeed'] = speed
    return dfSail
    
def speedAnalytics(speed):
    plt.figure()
    sns.distplot(speed, kde=False, norm_hist=True, hist_kws=dict(alpha=0.9))
    plt.show()
    
def speedAnalyticsYearly(df):
    year = []
    speed = []
    sY = datetime.datetime.utcfromtimestamp(df.iat[0,0]).year
    eY = datetime.datetime.utcfromtimestamp(df.iat[-1,0]).year
    print(sY)
    print(eY)
    
    for i in range(sY,eY+1):
        low = calendar.timegm(datetime.datetime(i,1,1,0,0).timetuple())
        high = calendar.timegm(datetime.datetime(i+1,1,1,0,0).timetuple())
        year.append(i)
        speed.append(df[(df['unixtime']>=low) & (df['unixtime']<high)]['sog'].values)
        
   
    ax = sns.boxplot(data=speed, color="lightsteelblue")
    ax.set_xticklabels(year)
    ax.set(xlabel='Year', ylabel='Speed [knots]')

                
    
    

