#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Feb 27 12:11:57 2019

@author: havard
"""
import numpy as np
import pandas as pd
import math
import matplotlib.pyplot as plt
import sqlite3 as sql
import time
import datetime
import calendar
import seaborn as sns
import matplotlib.pyplot as plt
import sklearn.cluster as cluster
from mpl_toolkits.basemap import Basemap

c1 = 'steelblue'
c2 = 'darkorange'
c3 = 'lightseagreen'
c1l = 'lightblue'
c2l = 'bisque'
c3l = 'paleturquoise'


def mapScatterGlobal(x,y):
    '''
    Plots scatter map for coordinates x(longitude), y(lattitude)
    '''
    minlon = -180
    minlat = -70
    maxlon = 180
    maxlat = 90
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20

    fig,ax = plt.subplots(figsize=(15,7))
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,
                urcrnrlat=maxlat,rsphere=(6378137.00,6356752.3142),
                resolution='l',projection='cyl',lat_0=lat0,lon_0=lon0,
                lat_ts=lat1)
    m.drawmapboundary(fill_color='white')
    m.fillcontinents(color='lightgrey',lake_color='white',zorder=1)
    m.scatter(x,y,5,marker='o',c='blue',alpha = 0.5,zorder=2)
    fig.tight_layout()


def mapScatterGlobalMsgType(df):
    '''
    Plots scatter map for coordinates x(longitude), y(lattitude)
    '''
    minlon = -180
    minlat = -70
    maxlon = 180
    maxlat = 90
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20

    fig,ax = plt.subplots(figsize=(15,7))
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,
                urcrnrlat=maxlat,rsphere=(6378137.00,6356752.3142),
                resolution='l',projection='cyl',lat_0=lat0,lon_0=lon0,
                lat_ts=lat1)
    m.drawmapboundary(fill_color='white')
    m.fillcontinents(color='lightgrey',lake_color='white',zorder=1)
    m.scatter(df[df.msgType==1].longitude,df[df.msgType==1].latitude,1,marker='o',c=c1,alpha = 0.4,zorder=2)
    m.scatter(df[df.msgType==2].longitude,df[df.msgType==2].latitude,1,marker='o',c=c3,alpha = 0.8,zorder=2)
    m.scatter(df[df.msgType==3].longitude,df[df.msgType==3].latitude,1,marker='o',c=c2,alpha = 0.8,zorder=2)
    fig.tight_layout()

def mapScatterLocal(x,y):
    '''
    Plots scatter map for coordinates x(longitude), y(lattitude)
    '''
    minlon = max(-180,min(x)-6)
    minlat = max(-90,min(y)-6)
    maxlon = min(180,max(x)+6)
    maxlat = min(90,max(y)+6)
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20

    fig,ax = plt.subplots(figsize=(15,7))
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,
                urcrnrlat=maxlat,rsphere=(6378137.00,6356752.3142),
                resolution='l',projection='cyl',lat_0=lat0,lon_0=lon0,
                lat_ts=lat1)
    m.drawmapboundary(fill_color='white')
    m.fillcontinents(color='lightgrey',lake_color='white',zorder=1)
    m.scatter(x,y,5,marker='o',c='blue',alpha = 0.5,zorder=2)
    fig.tight_layout()
    
    
def mapErroneousStatus(df):
    '''
    Plots scatter map for coordinates x(longitude), y(lattitude),
    '''
    minlon = -180
    minlat = -65
    maxlon = 180
    maxlat = 70
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20

    fig,ax = plt.subplots(figsize=(15,7))
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,
                urcrnrlat=maxlat,rsphere=(6378137.00,6356752.3142),
                resolution='l',projection='cyl',lat_0=lat0,lon_0=lon0,
                lat_ts=lat1)
    m.drawmapboundary(fill_color='white')
    m.fillcontinents(color='lightgrey',lake_color='white',zorder=1)
    
    df[df['nav_status']!=0]['nav_status'].value_counts()
    print('INcude Table with distinct vessel sending these messages')
    
    count = []
    
    df1 = df[(df['nav_status']==5)&(df['sog']>0.5)]
    count.append(df1.size)
    m.scatter(df1['longitude'],df1['latitude'],1,marker='o',c=c1,alpha = 1,zorder=2,label='Moored and Speed over 0.5: '+str(len(df1)))
    
    df1 = df[(df['nav_status']==0)&(df['sog']==0)]
    count.append(df1.size)
    m.scatter(df1['longitude'],df1['latitude'],1,marker='o',c=c2,alpha = 1,zorder=2,label='Underway sialing and speed =0: '+str(len(df1)))
    
    df1 = df[(df['nav_status']==1)&(df['sog']>1)]
    count.append(df1.size)
    m.scatter(df1['longitude'],df1['latitude'],1,marker='o',c=c3,alpha = 1,zorder=2,label='At anchor and Speed over 1: '+str(len(df1)))
    
    df1 = df[(df['nav_status']!=0)&(df['nav_status']!=1)&(df['nav_status']!=5)]
    m.scatter(df1['longitude'],df1['latitude'],1,marker='o',c=c1l,alpha = 0.5,zorder=2,label='Not status 0,1 or 5: '+str(len(df1)))
    
    print('Number of Errounous stauts = '+ str(sum(count))+' Persent of total : '+ str(sum(count)/df.size))
    
    #lgnd = plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,ncol=2, mode="expand", borderaxespad=0.)
    lgnd=ax.legend(loc='upper center', bbox_to_anchor=(0.5, -0.01),prop={'size': 11.5}, shadow=True,ncol=4)
    lgnd.draw_frame(False)
    lgnd.legendHandles[0]._sizes = [30]
    lgnd.legendHandles[1]._sizes = [30]
    lgnd.legendHandles[2]._sizes = [30]
    lgnd.legendHandles[3]._sizes = [30]


    fig.tight_layout()
    
    
def mapPlotDensity(df):
    '''
    Density plot for dataframe, df[2] = x,df[3] = y
    '''
    y = df['latitude']
    x = df['longitude']
#    minlon = max(-180,min(x)-10)
#    minlat = max(-90,min(y)-10)
#    maxlon = min(180,max(x)+10)
#    maxlat = min(90,max(y)+10)
    minlon = -180
    minlat = -65
    maxlon = 180
    maxlat = 80
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20

    fig,ax = plt.subplots(figsize=(15,7))
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,
                urcrnrlat=maxlat,rsphere=(6378137.00,6356752.3142),
                resolution='l',projection='cyl',lat_0=lat0,lon_0=lon0,
                lat_ts=lat1)
    m.drawmapboundary(fill_color='white')
    m.fillcontinents(color='lightgrey',lake_color='white',zorder=1)
    m.drawcountries(color='white')

    vessels = df['userid'].unique()
    for MML in vessels:
        df_temp = df[df['userid'] == MML].copy()
        df_diff = df_temp[['latitude','longitude']].diff().abs() #Plot only with dist close to eachoder
        df_temp[(df_diff['latitude'] > 5) | (df_diff['longitude'] > 5 )] = np.nan
        m.plot(df_temp['longitude'],df_temp['latitude'],0.01,c=c1,alpha = 0.02,zorder=2)
    plt.tight_layout()
    
    
def mapPlotDensityType(df,df2,df3):
    y = df['latitude']
    x = df['longitude']
    minlon = -180
    minlat = -65
    maxlon = 180
    maxlat = 70
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20

    fig,ax = plt.subplots(figsize=(15,7))
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,
                urcrnrlat=maxlat,rsphere=(6378137.00,6356752.3142),
                resolution='l',projection='cyl',lat_0=lat0,lon_0=lon0,
                lat_ts=lat1)
    m.drawmapboundary(fill_color='white')
    m.fillcontinents(color='lightgrey',lake_color='white',zorder=1)
    m.drawcountries(color='white')

    vessels = df['userid'].unique()
    for MML in vessels:
        df_temp = df[df['userid'] == MML].copy()
        df_diff = df_temp[['latitude','longitude']].diff().abs() #Plot only with dist close to eachoder
        df_temp[(df_diff['latitude'] > 5) | (df_diff['longitude'] > 5 )] = np.nan
        m.plot(df_temp['longitude'],df_temp['latitude'],0.02,c=c2,alpha = 0.02,zorder=2)
    
    vessels = df2['userid'].unique()
    for MML in vessels:
        df_temp = df2[df2['userid'] == MML].copy()
        df_diff = df_temp[['latitude','longitude']].diff().abs() #Plot only with dist close to eachoder
        df_temp[(df_diff['latitude'] > 5) | (df_diff['longitude'] > 5 )] = np.nan
        m.plot(df_temp['longitude'],df_temp['latitude'],0.02,c=c1,alpha = 0.02,zorder=2)
    
    vessels = df3['userid'].unique()
    for MML in vessels:
        df_temp = df3[df3['userid'] == MML].copy()
        df_diff = df_temp[['latitude','longitude']].diff().abs() #Plot only with dist close to eachoder
        df_temp[(df_diff['latitude'] > 5) | (df_diff['longitude'] > 5 )] = np.nan
        m.plot(df_temp['longitude'],df_temp['latitude'],0.02,c=c3,alpha = 0.02,zorder=2)
    plt.tight_layout()
    
    
    
def plotOceanPolygons(polygons):
    fig,ax = plt.subplots(figsize=(15,7))
    m = Basemap(rsphere=(6378137.00,6356752.3142),resolution='l',projection='cyl',lon_0=0)
    m.drawmapboundary(fill_color='white')
    m.fillcontinents(color='lightgrey',lake_color='white')
    for polygon in polygons:
        x,y = zip(*polygon)
        m.plot(x,y,markersize = 0)
        ax.fill(x,y,alpha=0.2)
    plt.show()
    
    
def plotWorldOceans(x,y,poly):
    '''
    Visualisation of polygon, centre in x,y and polygon
    '''
    minlon = max(-180,x-2)
    minlat = max(-90,y-2)
    maxlon = min(180,x+2)
    maxlat = min(90,y+2)
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20

    fig,ax = plt.subplots(figsize=(15,15))
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,
                urcrnrlat=maxlat,rsphere=(6378137.00,6356752.3142),
                resolution='l',projection='cyl',lat_0=lat0,lon_0=lon0,
                lat_ts=lat1)
    m.drawmapboundary(fill_color='white')
    m.fillcontinents(color='lightgrey',lake_color='white',zorder=1)
    x1,y1 = zip(*poly)
    m.plot(x1,y1,1,c = 'red',alpha = 0.3)
    m.plot([x1[-1],x1[0]],[y1[-1],y1[0]],1,c = 'red',alpha = 0.2,zorder = 2)
    m.scatter(x1,y1,2,c = 'red',alpha = 0.5,zorder = 2)
    m.scatter(x,y,10,c = 'red',marker = 'o',alpha = 1,zorder = 2)
    ax.fill(x1,y1,c = 'blue',alpha=0.1)
    
    
    
def plotWorldOceans(polygons):
    '''
    Visualisation of polygon, centre in x,y and polygon
    '''
    minlon = -180
    minlat = -65
    maxlon = 180
    maxlat = 75
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20

    fig,ax = plt.subplots(figsize=(15,15))
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,
                urcrnrlat=maxlat,rsphere=(6378137.00,6356752.3142),
                resolution='l',projection='cyl',lat_0=lat0,lon_0=lon0,
                lat_ts=lat1)
    m.drawmapboundary(fill_color='white')
    m.fillcontinents(color='lightgrey',lake_color='white',zorder=1)
    #m.drawcountries(color='white')
    colors =[c1,c1,c2,c3,c1l,c2l,c3l,c3l]
    for polygon,color in zip(polygons,colors):
        x1,y1 = zip(*polygon)
        m.plot(x1,y1,1,c = color,alpha = 0.3)
        m.plot([x1[-1],x1[0]],[y1[-1],y1[0]],1,c = color,alpha = 0.2,zorder = 2)
        ax.fill(x1,y1,c = color,alpha=0.5)
        
    
    
def plotLegNr(df):
    fig,ax = plt.subplots(figsize=(15,7))
    m = Basemap(rsphere=(6378137.00,6356752.3142),resolution='l',projection='cyl',lon_0=0)
    m.drawmapboundary(fill_color='white')
    m.fillcontinents(color='lightgrey',lake_color='white')
    legNr = df.Leg.unique()
    df = df[df.Leg!=0]
    for leg in legNr:
        m.scatter(df[df.Leg == leg].longitude,df[df.Leg == leg].latitude,1,alpha = 1)
    plt.show()    


def mapStatusScatterLocal(df):
    '''
    Plots scatter map for coordinates x(longitude), y(lattitude)
    '''
    x = df.longitude
    y = df.latitude
    minlon = max(-180,min(x)-6)
    minlat = max(-90,min(y)-6)
    maxlon = min(180,max(x)+6)
    maxlat = min(90,max(y)+6)
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20


    fig,ax = plt.subplots(figsize=(15,15))
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,
                urcrnrlat=maxlat,rsphere=(6378137.00,6356752.3142),
                resolution='l',projection='cyl',lat_0=lat0,lon_0=lon0,
                lat_ts=lat1)
    m.drawmapboundary(fill_color='white')
    m.fillcontinents(color='lightgrey',lake_color='white',zorder=1)
    print('New')
    m.scatter(df[df.status==1].longitude,df[df.status==1].latitude,1,marker='o',c=c3,alpha = 0.1,zorder=2,label = 'Sailing')
    m.scatter(df[df.status==2].longitude,df[df.status==2].latitude,5,marker='o',c=c1,alpha = 0.8,zorder=2,label = 'In Port')
    m.scatter(df[df.status==3].longitude,df[df.status==3].latitude,5,marker='o',c=c2,alpha = 0.8,zorder=2,label = 'Anchor')
#    m.scatter(df[df.nav_status==0].longitude,df[df.nav_status==0].latitude,1,marker='o',c=c3,alpha = 0.1,zorder=2,label = 'Sailing')
#    m.scatter(df[df.nav_status==5].longitude,df[df.nav_status==5].latitude,5,marker='o',c=c1,alpha = 0.8,zorder=2,label = 'In Port')
#    m.scatter(df[df.nav_status==1].longitude,df[df.nav_status==1].latitude,5,marker='o',c=c2,alpha = 0.8,zorder=2,label = 'Anchor')
    fig.tight_layout()
    
    
    lgnd=ax.legend(loc='upper center', bbox_to_anchor=(0.5, -0.01),prop={'size': 11.5}, shadow=True,ncol=3)
    lgnd.draw_frame(False)
    lgnd.legendHandles[0]._sizes = [100]
    lgnd.legendHandles[1]._sizes = [100]
    lgnd.legendHandles[2]._sizes = [100]
