#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Feb  5 14:00:09 2019

@author: havard
"""

#Import Packages
import sqlite3 as sql
import time
import datetime
import calendar
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd

plt.style.use('seaborn-whitegrid')
c1 = 'steelblue'
c2 = 'darkorange'
c3 = 'lightseagreen'
c1l = 'lightblue'
c2l = 'bisque'
c3l = 'paleturqoise'



#Return df with messagecount pr vessel
#Message Frequency Per ship

def msgCountPrVessel(dbPath,table):
    query = "SELECT userid, count(*) FROM %s GROUP BY userid ORDER BY count(*)" %(table)
    con = sql.connect(dbPath)
    with con:
        ais_dataframe = pd.read_sql_query(query,con)
    con.close()
    
    ais_dataframe.columns = ['userid','count']
    
    return ais_dataframe

#EXTRACT RELEVANT DATA FROM DATABASE (Message Type 1)
def Extract1(dbPath,table,fromTime,toTime):
    '''
    Extracts info given f(dbpath,table,fromTime,toTime,fromSpe,toSpeed)
    '''
    tic = time.time()
    conn = sql.connect(dbPath)
    SQLstring = "SELECT unixtime, sog, latitude, longitude, userid, nav_status,\
        cog,msgType FROM %s WHERE unixtime >= %s and unixtime <= %s ORDER BY \
        UNIXTIME ASC" % (table, str(fromTime), str(toTime))
    with conn:
        df = pd.read_sql_query(SQLstring,conn)
    conn.close()
    print("Message Type 1 extraction time= ",time.time()-tic)
    return df

#EXTRACT RELEVANT DATA FROM DATABASE (Message Type 5)
def Extract5(dbPath,table,fromTime,toTime):
    tic = time.time()
    conn = sql.connect(dbPath)
    SQLstring = "SELECT unixtime,dest,imo,ship_type,\
        userid,draught,msgType FROM %s WHERE unixtime >= %s and unixtime <= %s ORDER BY \
        UNIXTIME ASC" % (table, str(fromTime), str(toTime))
        
    with conn:
        df = pd.read_sql_query(SQLstring,conn)
    conn.close()
    print("Message Type 5 extraction time= ",time.time()-tic)
    return df

def DatabaseInfo(dbPath,table):
    '''
    Function for displaying database statisitcs, f(dbpath,table)
    '''
    st,et,count,unique = Statistics(dbPath,table)
    sT = datetime.datetime.utcfromtimestamp(st)
    eT = datetime.datetime.utcfromtimestamp(et)
    print(sT)
    print(eT)
    sY = datetime.datetime.utcfromtimestamp(st).year
    eY = datetime.datetime.utcfromtimestamp(et).year
    countYearly, uniqueYearly = StatisticsPrYear(dbPath,table,sY,eY)
        
    years = list(range(sY,eY+1))
    df = pd.DataFrame()
        
#    df[0] = years
#    df[1] = countYearly
#    df[2] = uniqueYearly
    
    print(years)
    print(countYearly)
    print(uniqueYearly)
    
    fig, ax1 = plt.subplots()
    ax1.set_xlabel('Year')
    ax1.set_ylabel('Number of Messages', color = c1)
    ax1.bar(np.arange(len(years)),countYearly,width=0.8,color=c1)
    ax1.set_xticks(range(len(years)))
    ax1.set_xticklabels(years)
    ax2 = ax1.twinx()
    ax2.set_ylabel('Number of Unique Vessels',color=c2)
    ax2.scatter(np.arange(len(years)),uniqueYearly,c=c2)
#    for i,txt in enumerate(uniqueYearly):
#        ax2.annotate(txt,(i,countYearly[i]))
    for i in np.arange(len(years)):
        ax2.annotate(uniqueYearly[i],(i+.1,uniqueYearly[i]))
    ax2.grid(False)
    fig.tight_layout() 
    plt.show()
    '''
    fig = plt.figure() # Create matplotlib figure
    ax = fig.add_subplot(111) # Create matplotlib axes
    ax2 = ax.twinx() # Create another axes that shares the same x-axis as ax.
    #width = 0.4
  
    #df[1].plot(kind='bar', color= c1, ax=ax, width=width, position=1, alpha = 0.7)
    #df[2].plot(kind='bar', color= c2, ax=ax2, width=width, position=0, alpha = 0.7)
    
    ax.set_ylabel('Number of Messages',color = c1)
    ax.set_xlabel('Year')
    ax.set_xticklabels(df[0])
    ax2.set_ylabel('Number of Unique Vessels',color = c2)
    ax2.grid(False)
    fig.tight_layout() 
    plt.show()
    '''  
#Statisics for whole table    
def Statistics(dbPath,table):
    test = sql.connect(dbPath).cursor()
    SQLstring = "SELECT COUNT(*) from %s" % (table)
    count = test.execute(SQLstring).fetchall()
    SQLstring = "SELECT MIN(unixtime) from %s" % (table)
    st = test.execute(SQLstring).fetchall()
    SQLstring = "SELECT MAX(unixtime) from %s" % (table)
    et = test.execute(SQLstring).fetchall()
    SQLstring = "SELECT COUNT(DISTINCT userid)  from %s" % (table)
    unique = test.execute(SQLstring).fetchall()
    test.close()
    return st[0][0],et[0][0],count[0][0],unique[0][0]

#Statistics for table pr. year
def StatisticsPrYear(dbPath,table,sY,eY):
    countYearly =[]
    uniqueYearly = []
    test = sql.connect(dbPath).cursor()
    for i in range(sY,eY+1):
        a = i
        b = i+1
        # convert the dates to unixtime
        fromTime = calendar.timegm(datetime.datetime(a,1,1,0,0).timetuple())
        toTime = calendar.timegm(datetime.datetime(b,1,1,0,0).timetuple())
     
        SQLstring = "SELECT COUNT(*) from %s WHERE\
        unixtime >= %s and unixtime <= %s" % (table,fromTime,toTime)
        count = test.execute(SQLstring).fetchall()
        SQLstring = "SELECT COUNT(DISTINCT userid) from %s WHERE\
        unixtime >= %s and unixtime <= %s" % (table,fromTime,toTime)
        countU = test.execute(SQLstring).fetchall()
        
        countYearly.append(count[0][0])
        uniqueYearly.append(countU[0][0])   
    test.close()
    return countYearly,uniqueYearly
