#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Mar  7 09:33:47 2019

@author: havard
"""
import time
import datetime
import calendar
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from mpl_toolkits.basemap import Basemap


def generatePolygons():
    '''
    Generates the oceanc poylgons, and return list containg them
    '''
    polygons = list()
    westPacific = [[-180,90],[-100,90],[-103,22],[-76,7],[-61,-19],
                    [-80,-90],[-180,-90]] 
    eastPacific = [[180,90],[101,90],[101,15],[145,-29],[145,-90],[180,-90]]
    
    atlantic = [[25,-90],[32,30.5],[37,32],[37,90],[-100,90],
                        [-103,22],[-76,7],[-61,-19],[-80,-90],[25,-90]]
    indian = [[25,-90],[32,30.5],[37,32],[37,90],[101,90],[101,15],[145,-29],[145,-90],[25,-90]]
        
    polygons.append(westPacific)
    polygons.append(eastPacific)
    polygons.append(atlantic)
    polygons.append(indian)


    return polygons

def pInsidePoly(x,y,polygon):
    '''
    Check if point insde polygon, using Ray-Casting algorithm
    (Longitude,Latityde,polygon)
    return true if insde, else false
    '''
    n = len(polygon)
    result = False
    x1,y1 = polygon[0]
    for i in range(n+1):
        x2,y2 = polygon[i % n]
        if y>min(y1,y2):
            if y <= max(y1,y2):
                if x <= max(x1,x2):
                    if y1 != y2:
                        xinters = (y-y1)*(x2-x1)/(y2-y1)+x1
                    if x1 == x2 or x <= xinters:
                        result = not result
        x1,y1 = x2,y2
    return result



def dfInsidePoly(dfI,polygon):
    '''
    Check if Datafarames points is 
    (DataFrame, polygon)
    Returns a df with points inside the polygon
    '''    
    #Minimize search
    x,y = zip(*polygon)
    max_x,min_x,max_y,min_y = max(x),min(x),max(y),min(y)
    dfIn = dfI[(dfI['longitude'] < max_x) & (dfI['longitude'] > min_x) & 
               (dfI['latitude'] < max_y) & (dfI['latitude'] > min_y)].copy()
        
    #Defining new colum in dataframe
    dfIn['inside'] = False
    
    for i in dfIn.index:
        if pInsidePoly(dfIn['longitude'][i],dfIn['latitude'][i],polygon):
            dfIn.at[i,'inside'] = True        
        
    dfIn = dfIn[dfIn.inside == True]
    return dfIn


def lableArea(df):
    df['Area'] = 'init'
    polygons = generatePolygons()
    lables =['Pac','Pac','Atl','Ind']
    
    for polygon,lable in zip(polygons,lables):
        
        def rayCast(row,polygon):
            if row.Area == 'init':
                x,y = zip(*polygon)
                max_x,min_x,max_y,min_y = max(x),min(x),max(y),min(y)
                if (row.longitude<max_x) & (row.longitude>min_x) & (row.latitude<max_y) & (row.latitude>min_y):
                    if pInsidePoly(row.longitude,row.latitude,polygon):
                        return lable
            return row.Area
        x,y = zip(*polygon)
        max_x,min_x,max_y,min_y = max(x),min(x),max(y),min(y)
        #df.loc[(df.longitude<max_x)&(df.longitude > min_x)&
        #     (df.latitude<max_y)&(df.latitude>min_y),'Area'].apply(rayCast,args=(polygon,),axis=1)
               
               #apply(rayCast,args=(polygon,),axis=1)
        
        df['Area'] = df.apply(rayCast,args=(polygon,),axis=1)
    return df
    

#For visualizaton of Geofencing
def lableExtremal(df,polygon):
    x,y = zip(*polygon)
    max_x,min_x,max_y,min_y = max(x),min(x),max(y),min(y)
    df.loc[(df.longitude<max_x)&(df.longitude > min_x)&
             (df.latitude<max_y)&(df.latitude>min_y),'location']=1
    
    def lableInside(row,polygon):
        if pInsidePoly(row.longitude,row.latitude,polygon):
            return 2
        return row.location
    
    df['location'] = df.apply(lableInside,args=(polygon,),axis=1)  
    return df
    
def visualizeGeoFence(df):
    df['location'] = 0
    
    polygons = generatePolygons()
    
    df = lableExtremal(df,polygons[0])
    df = lableExtremal(df,polygons[1])
    
    c1 = 'steelblue'
    c2 = 'darkorange'
    c3 = 'lightseagreen'
    c1l = 'lightblue'
    c2l = 'bisque'
    c3l = 'paleturquoise'
    
    minlon = -180
    minlat = -65
    maxlon = 180
    maxlat = 75
    lat0 = (maxlat+minlat)/2
    lon0 = (maxlon+minlon)/2
    lat1 = (maxlat+minlat)/2-20

    fig,ax = plt.subplots(figsize=(15,7))
    m = Basemap(llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon,
                urcrnrlat=maxlat,rsphere=(6378137.00,6356752.3142),
                resolution='l',projection='cyl',lat_0=lat0,lon_0=lon0,
                lat_ts=lat1)
    m.drawmapboundary(fill_color='white')
    m.fillcontinents(color='lightgrey',lake_color='white',zorder=1)
    m.scatter(df[df.location==1].longitude,df[df.location==1].latitude,2,marker='o',c=c2,alpha = 0.5,zorder=2)
    m.scatter(df[df.location==2].longitude,df[df.location==2].latitude,2,marker='o',c=c1,alpha = 0.5,zorder=2)
    m.scatter(df[df.location==0].longitude,df[df.location==0].latitude,2,marker='o',c='black',alpha = 0.2,zorder=2)
    
    for polygon in polygons[:2]:
        x1,y1 = zip(*polygon)
        m.plot(x1,y1,1,c = c1,alpha = 0.3)
        m.plot([x1[-1],x1[0]],[y1[-1],y1[0]],1,c = c1,alpha = 0.2,zorder = 2)
        ax.fill(x1,y1,c = c1,alpha=0.5)
      
    fig.tight_layout()       
    return df 
    
    